rm(list=ls())

library(swaRm)
library(plyr)
library(data.table)
library(sp)
library(lubridate)
library(trackdf)

# Load in dataframe

setwd()
load("GPS.RData")

GPS$time <- as.POSIXct(GPS$time, "%Y-%m-%d %H:%M:%S", tz = "UTC")
GPS$min <- floor_date(GPS$time, unit="minute")
GPS$date <- as.Date(GPS$time)

GPSmin <- ddply(.data = GPS, 
                .variables = c("id"),  
                .fun = function(x) { 
                  min <- x[!duplicated(x$min),]
                  dat = data.frame(min) 
                  return(dat) 
                },
                .progress = "text") 

GPSmin <- GPSmin[c("id","min","x","y","area", "activecollars", "activecollars_area")]

detach("package:plyr", unload=TRUE)
library(dplyr)

df = split(GPSmin, list(GPSmin$id))

trajs <- lapply(names(df), function(x) {
  
  
  t_df = trackdf::track(x = df[[x]]$x, y = df[[x]]$y, id = df[[x]]$id, 
                        units = "mins",
                        t = df[[x]]$min,
                        area = df[[x]]$area,
                        tz = "UTC",
                        table = "df",
                        geo = FALSE) %>%
    mutate(linDist = linDist(x, y, geo = FALSE)) %>%
    mutate(cumDist = cumsum(linDist)) %>%
    mutate(linSpeed = linSpeed(x, y, t, geo = FALSE),
           linAcc = linAcc(x, y, t, geo = FALSE),
           heading = heading(x, y, geo = FALSE),
           angSpeed = angSpeed(x, y, t, geo = FALSE),
           angAcc = angAcc(x, y, t, geo = FALSE))
}) %>% trackdf::rbind_track()

trajs = group_by(trajs, t) %>%
  mutate(distToCentroid = dist2centroid(x, y, geo = FALSE)) %>%
  ungroup()

trajs <- group_by(trajs, t) %>%
  mutate(nearNeighbor = nn(x, y, id, geo = FALSE),
         nearNeighborDist = nnd(x, y, geo = FALSE))

trajSummary <- group_by(trajs, t) %>%
  summarise(centroid(x, y, geo = FALSE)) %>%
  ungroup()

trajSummary <- group_by(trajs, t) %>% 
  summarize(distToCentroid = mean(distToCentroid)) 

trajSummary <- group_by(trajs, t) %>% 
  summarize(nearNeighborDist = mean(nearNeighborDist)) %>%
  merge(trajSummary, by = c("t"))

trajSummary <- group_by(trajs, t) %>%
  summarise(meanHeading = mean(heading),
            seHeading = sd(heading)/sqrt(length(heading)),
            sdHeading = sd(heading),
            polarization = polOrder(heading)) %>%
  merge(trajSummary, by = c("t"))

trajSummary <- group_by(trajs, t) %>%
  summarise(meanSpeed = mean(linSpeed),
            seSpeed = sd(linSpeed)/sqrt(length(linSpeed)))%>%
  merge(trajSummary, by = c("t"))

trajSummary <- group_by(trajs, t) %>% 
  summarize(sphericity = sphericity(x, y)) %>%
  merge(trajSummary, by = c("t"))

trajSummary <- group_by(trajs, t) %>% 
  summarize(stretch = stretch(x, y)) %>%
  merge(trajSummary, by = c("t"))

trajSummary <- group_by(trajs, t) %>% 
  summarize(chullPerim = chullPerimeter(x, y, geo = FALSE),
            chullArea = chullArea(x, y, geo = FALSE))%>%
  merge(trajSummary, by = c("t"))


trajSummary <- as.data.frame(trajSummary)

trajSummary$t <- as.character(trajSummary$t)

GPSmin$min <- as.character(GPSmin$min)

GPSmin <-GPSmin[!duplicated(GPSmin$min),]

CollectiveParameters <- merge(trajSummary, 
                         GPSmin[c("min", "activecollars", "activecollars_area", "area")], 
                         by.x = c("t"), 
                         by.y = c("min"),
                         all.x=T)

CollectiveParameters <- CollectiveParameters[CollectiveParameters$activecollars >= 10,]
CollectiveParameters <- CollectiveParameters[CollectiveParameters$activecollars_area == CollectiveParameters$activecollars,]
CollectiveParameters$hour <- substr(CollectiveParameters$t, 12, 13)
CollectiveParameters <- CollectiveParameters[CollectiveParameters$hour < 16,]

table(CollectiveParameters$area)
